
#ifndef MComponentImpl_Header
#define MComponentImpl_Header

#include <MRefCountedPtr.h>
#include <gui/MShiftState.h>

#include <string>
#include <vector>

#include <qwidget.h>
#include <qcursor.h>


namespace Aztec {

  class MComponent;
  typedef MRefCountedPtr<MComponent> MComponentPtr;

  class MContainer;
  typedef MRefCountedPtr<MContainer> MContainerPtr;

  class AZTECGUI_EXPORT MComponent : public virtual Aztec::IMComponent {
  public:
    MComponent();
    MComponent(const std::string &name);
    ~MComponent();

    /**
     * This creates the implementation window
     */
    virtual bool createImpl();

    // MComponent methods.
    void setName(const std::string name);
    std::string getName();
    void setParent(MContainerPtr parent);
    MContainerPtr getParent();
    void setVisible(bool visible);
    void setPosition(const MPoint2D &pos);
    void setPosition(int X, int Y);
    void setPosition(int X, int Y, int width, int height);
    MPoint2D getPosition();
    void setSize(const MSize2D &size);
    void setSize(int width, int height);
    MRect2D getWindowRect();
    void setWindowRect(const MRect2D &rect);
    MSize2D getClientSize();
    MSize2D getSize();
    MSize2D getMinimumSize();
    void setMinimumSize(const MSize2D &size);
    bool isMinimumSizeSet();
    MSize2D getPreferredSize();
    void setPreferredSize(const MSize2D &size);
    bool isPreferredSizeSet();
    void refresh();
    void setEnabled(bool enabled);
    void setMouseCapture(bool capturing);
    void setMouseCursor(MMouseCursor cursor);
    void setBackgroundColour(const MColour &colour);
    void setForegroundColour(const MColour &colour);

    virtual QWidget* getHandle();

    bool onMouseMove(const MMouseEvent &event);
    bool onMousePressed(const MMouseEvent &event);
    bool onMouseReleased(const MMouseEvent &event);
    bool onMouseDoubleClick(const MMouseEvent &event);
    bool onMouseWheel(const MMouseEvent &event);

    bool onResize(int newWidth, int newHeight);
    bool onShow();
    bool onPaint();
    void onCreate();

    bool onKeyPressed(const MKeyEvent &event);
    bool onKeyReleased(const MKeyEvent &event);

    void addMouseEventListener(const MMouseEventListenerPtr &listener);
    void addKeyEventListener(const MKeyEventListenerPtr &listener);

    bool handleKeyEvent(const MKeyEvent &event);

  protected:
    std::string m_Name;

    /**
     * This is the parent to this component. Note
     * we do not use a reference counted pointer here
     * because of the circular reference this would create.
     */
    MContainer* m_Parent;

    /**
     * This is the preferred size of the component
     */
    MSize2D m_PreferredSize;
    
    /**
     * This is the minimum size of the component
     */
    MSize2D m_MinimumSize;

    /**
     * This is a pointer to our internal Qt Widget
     */
    QWidget* m_Handle;

    bool bgColourSet;
    MColour bgColour;
    bool fgColourSet;
    MColour fgColour;

    std::vector<MMouseEventListenerPtr> mouseListeners;
    std::vector<MKeyEventListenerPtr> keyListeners;

    void fireMouseListeners(const MMouseEvent &event);
    void fireKeyListeners(const MKeyEvent &event);
  };

}


#endif

